@@ -708,7 +708,7 @@ hildon_desktop_popup_menu_release_event (GtkWidget      *widget,
 
   return TRUE;
 } 
-	
+
 static gboolean 
 hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
                                        GdkEventButton *event)
@@ -722,6 +722,9 @@ hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
 
   for (l = menu_items; l != NULL; l = g_list_next (l))
   {
+    if (!hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)))
+      continue;
+
     gtk_widget_get_pointer (GTK_WIDGET (l->data), &x, &y);
 
     w = GTK_WIDGET (l->data)->allocation.width;
@@ -731,8 +734,7 @@ hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
     {
       if (GTK_IS_MENU_ITEM (l->data))
       {
-        if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)) &&
-            menu->priv->selected_item != GTK_MENU_ITEM (l->data))
+        if (menu->priv->selected_item != GTK_MENU_ITEM (l->data))
         {
           if (GTK_IS_ITEM (menu->priv->selected_item))
             gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
