@@ -563,6 +563,75 @@ hd_keys_shortcut_free (HDKeyShortcut *shortcut)
   g_free (shortcut);
 }
 
+/* Check whether Fn is required on the built-in keyboard to obtain
+ * this keysym with this keycode. We need to check that Fn + keycode
+ * gives keysym, but also that keycode alone does not give keysym. Also
+ * we need to perform the same check with Shift */
+static gboolean
+hd_key_shortcut_needs_fn (GdkKeymapKey *key, gint keysym)
+{
+  gint level, group;
+  guint keyval;
+  GdkModifierType consumed_modifiers;
+  gboolean with_fn = FALSE, without_fn = FALSE;
+  gboolean with_shift_fn = FALSE, without_shift_fn = FALSE;
+
+  if (gdk_keymap_translate_keyboard_state (NULL,
+                                           key->keycode,
+                                           GDK_MOD5_MASK,
+                                           key->group,
+                                           &keyval,
+                                           &group,
+                                           &level,
+                                           &consumed_modifiers))
+  {
+    if ((level == key->level) && (keyval == keysym) && (group == key->group))
+      with_fn = TRUE;
+  }
+
+  if (gdk_keymap_translate_keyboard_state (NULL,
+                                           key->keycode,
+                                           0,
+                                           key->group,
+                                           &keyval,
+                                           &group,
+                                           &level,
+                                           &consumed_modifiers))
+  {
+    if ((level == key->level) && (keyval == keysym) && (group == key->group))
+      without_fn = TRUE;
+  }
+
+  if (gdk_keymap_translate_keyboard_state (NULL,
+                                           key->keycode,
+                                           GDK_MOD5_MASK | GDK_SHIFT_MASK,
+                                           key->group,
+                                           &keyval,
+                                           &group,
+                                           &level,
+                                           &consumed_modifiers))
+  {
+    if ((level == key->level) && (keyval == keysym) && (group == key->group))
+      with_shift_fn = TRUE;
+  }
+
+  if (gdk_keymap_translate_keyboard_state (NULL,
+                                           key->keycode,
+                                           GDK_SHIFT_MASK,
+                                           key->group,
+                                           &keyval,
+                                           &group,
+                                           &level,
+                                           &consumed_modifiers))
+  {
+    if ((level == key->level) && (keyval == keysym) && (group == key->group))
+      without_shift_fn = TRUE;
+  }
+
+  return (with_fn && !without_fn) || (with_shift_fn && !without_shift_fn);
+
+}
+
 static gboolean
 hd_key_shortcut_grab (HDKeysConfig  *keys, 
 		      HDKeyShortcut *shortcut,
@@ -578,19 +647,40 @@ hd_key_shortcut_grab (HDKeysConfig  *keys,
   for (i_keycode = 0; i_keycode < shortcut->n_keycodes; i_keycode++)
   {
     while (ignored_mask < (int) keys->lock_mask)
-    {                                       
+    {
+      gboolean  needs_fn;
+      guint     mask;
+
       if (ignored_mask & ~(keys->lock_mask))
       {
         ++ignored_mask;
         continue;
       }
 
-      
+      /* ignore shortcuts from a group different than the default one,
+       * as AFAIK there is no way to translate the group information
+       * into a keycode + modifier required by XGrabKey */
+      if (shortcut->keycodes[i_keycode].group != 0)
+      {
+        ignored_mask ++;
+        continue;
+      }
+
+      mask = shortcut->mod_mask | ignored_mask;
+
+
+      needs_fn = (shortcut->keycodes[i_keycode].level != 0 &&
+                  hd_key_shortcut_needs_fn (&shortcut->keycodes[i_keycode],
+                                            shortcut->keysym));
+
+      if (needs_fn)
+        mask |= Mod5Mask;
+
       if (ungrab)
       {
         XUngrabKey (GDK_DISPLAY(),
-	            shortcut->keycodes[i_keycode].keycode, 
-		    shortcut->mod_mask | ignored_mask,
+	            shortcut->keycodes[i_keycode].keycode,
+		    mask,
 		    GDK_ROOT_WINDOW());
       } 
       else 
@@ -601,7 +691,7 @@ hd_key_shortcut_grab (HDKeysConfig  *keys,
 	  
         XGrabKey (GDK_DISPLAY(),
 		  shortcut->keycodes[i_keycode].keycode,
-		  shortcut->mod_mask | ignored_mask,
+		  mask,
 		  GDK_ROOT_WINDOW(), 
 		  False, 
 		  GrabModeAsync, 
@@ -656,7 +746,6 @@ hd_keys_load_and_grab_all_shortcuts (HDKeysConfig *keys)
 	  
       if ((shortcut = hd_keys_shortcut_new (keys, key_def_str, i)) != NULL)
       {
-        hd_wm_debug ("Grabbing '%s'", key_def_str);
         hd_key_shortcut_grab (keys, shortcut, FALSE); 
         keys->shortcuts = g_slist_append (keys->shortcuts, shortcut);
       }
@@ -788,17 +877,30 @@ hd_keys_handle_keypress (HDKeysConfig *keys,
   {
     HDKeyShortcut *shortcut = (HDKeyShortcut *)item->data;
     KeySym keysym;
+    gint level;
+    guint levelled_keysym;
   
     gdk_error_trap_push ();
    
     keysym = XKeycodeToKeysym(GDK_DISPLAY(),keycode,shortcut->index);
 
     gdk_flush ();
-    
+
+    /* also check if this mod_mask gives a keysym with another level */
+    gdk_keymap_translate_keyboard_state (NULL,
+                                         keycode,
+                                         mod_mask,
+                                         0,
+                                         &levelled_keysym,
+                                         NULL,
+                                         &level,
+                                         NULL);
+
     if (gdk_error_trap_pop ())
       continue;
 		    
-    if (shortcut->mod_mask == mod_mask && shortcut->keysym == keysym)
+    if ((shortcut->mod_mask == mod_mask && shortcut->keysym == keysym) ||
+        (level != 0 && shortcut->keysym == levelled_keysym))
     {
       return shortcut;
     }
