@@ -330,6 +330,12 @@ hd_wm_cad_compare_items (gconstpointer a, gconstpointer b)
   return (ib->vmdata - ia->vmdata);
 }
 
+static void
+hd_wm_cad_item_remove (HDWMCADItem *item)
+{
+  item->win = NULL;
+}
+
 static gboolean
 hd_wm_prepare_close_application_dialog (HDWM *hdwm, HDWMCADAction action, gboolean *retval)
 {
@@ -440,8 +446,12 @@ hd_wm_prepare_close_application_dialog (HDWM *hdwm, HDWMCADAction action, gboole
     item->win = win;
     item->vmdata = hd_wm_get_vmdata_for_pid(pid);
     item->pid = pid;
+    g_object_weak_ref (G_OBJECT (win),
+                       (GWeakNotify)hd_wm_cad_item_remove,
+                       item);
       
     items = g_list_append(items, item);
+
   }
 
   if (g_list_length (items) == 0)
@@ -466,6 +476,18 @@ hd_wm_prepare_close_application_dialog (HDWM *hdwm, HDWMCADAction action, gboole
 
   g_signal_emit_by_name (hdwm, "close-app", action, items, retval);
 
+  for (l = items; l; l = l->next)
+  {
+    HDWMCADItem *item = l->data;
+    if (item->win)
+      g_object_weak_unref (G_OBJECT (item->win),
+                           (GWeakNotify) hd_wm_cad_item_remove,
+                           item);
+
+    g_free (item);
+  }
+  g_list_free (items);
+
   return TRUE;
 }
 
